package com.seeyon.apps.govdoc.util;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.logging.Log;

import com.seeyon.apps.collaboration.vo.SeeyonPolicy;
import com.seeyon.apps.edoc.enums.EdocEnum;
import com.seeyon.apps.govdoc.constant.GovdocEnum.TransferStatus;
import com.seeyon.apps.govdoc.po.GovdocExchangeDetail;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.ModuleType;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.constants.ApplicationSubCategoryEnum;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.content.mainbody.MainbodyType;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.taglibs.functions.Functions;
import com.seeyon.ctp.common.workflowmanage.vo.WorkflowData;
import com.seeyon.ctp.form.util.Enums.FormType;
import com.seeyon.ctp.form.bean.FormBean;
import com.seeyon.ctp.form.bean.FormFieldBean;
import com.seeyon.ctp.form.manager.GovdocFormExtendManager;
import com.seeyon.ctp.form.manager.GovdocFormOpinionSortManager;
import com.seeyon.ctp.form.po.GovdocFormExtend;
import com.seeyon.ctp.form.po.GovdocFormOpinionSort;
import com.seeyon.ctp.form.util.FormConstant;
import com.seeyon.ctp.organization.bo.V3xOrgUnit;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.IdentifierUtil;
import com.seeyon.ctp.util.SQLWildcardUtil;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.util.XMLCoder;
import com.seeyon.ctp.workflow.exception.BPMException;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import com.seeyon.v3x.edoc.manager.EdocSummaryManager;
import com.seeyon.v3x.edoc.webmodel.FormOpinionConfig;

import net.joinwork.bpm.definition.BPMSeeyonPolicy;

/**
 * 新公文工具类
 * 
 * @author 唐桂林
 * 
 */
public class GovdocUtil {

	private static final Log log = CtpLogFactory.getLog(GovdocUtil.class);
	private static WorkflowApiManager wapi;
	private static EdocSummaryManager edocSummaryManager;
	private static final int INENTIFIER_SIZE = 20;

	private static OrgManager orgManager;

	/**
	 * 新老公文edocType与govdocType互转
	 * 
	 * @param edocType
	 * @return
	 */
	public static String getCategoryBySubApp(int subApp, String configItem) {
		if (subApp == ApplicationSubCategoryEnum.edoc_fawen.key()) {
			return EnumNameEnum.edoc_new_send_permission_policy.name();
		} else if (subApp == ApplicationSubCategoryEnum.edoc_shouwen.key()) {
			return EnumNameEnum.edoc_new_rec_permission_policy.name();
		} else if (subApp == ApplicationSubCategoryEnum.edoc_qianbao.key()) {
			return EnumNameEnum.edoc_new_qianbao_permission_policy.name();
		} else if (subApp == ApplicationSubCategoryEnum.edoc_jiaohuan.key()) {
			if ("zhihui".equals(configItem)) {
				return EnumNameEnum.edoc_new_rec_permission_policy.name();
			}
			return EnumNameEnum.edoc_new_change_permission_policy.name();
		} else if (subApp == ApplicationSubCategoryEnum.old_edocSend.key()) {
			return EnumNameEnum.edoc_send_permission_policy.name();
		} else if (subApp == ApplicationSubCategoryEnum.old_edocRec.key()) {
			return EnumNameEnum.edoc_rec_permission_policy.name();
		} else if (subApp == ApplicationSubCategoryEnum.old_edocSign.key()) {
			return EnumNameEnum.edoc_qianbao_permission_policy.name();
		}
		return EnumNameEnum.edoc_new_send_permission_policy.name();
	}

	/**
	 * 新老公文edocType与govdocType互转
	 * 
	 * @param edocType
	 * @return
	 */
	public static Integer getGovdocTypeByEdocType(int edocType) {
		int govdocType = 0;
		if (edocType == 0) {
			govdocType = ApplicationSubCategoryEnum.edoc_fawen.key();
		} else if (edocType == 1) {
			govdocType = ApplicationSubCategoryEnum.edoc_shouwen.key();
		} else if (edocType == 2) {
			govdocType = ApplicationSubCategoryEnum.edoc_qianbao.key();
		}
		return govdocType;
	}

	/**
	 * 新老公文edocType与govdocType互转
	 * 
	 * @param govdocType
	 * @return
	 */
	public static Integer getEdocTypeByGovdoc(int govdocType) {
		int edocType = 0;
		if (govdocType == ApplicationSubCategoryEnum.edoc_fawen.key()) {
			edocType = 0;
		} else if (govdocType == ApplicationSubCategoryEnum.edoc_shouwen.key()) {
			edocType = 1;
		} else if (govdocType == ApplicationSubCategoryEnum.edoc_qianbao.key()) {
			edocType = 2;
		} else if (govdocType == ApplicationSubCategoryEnum.edoc_jiaohuan.key()) {
			edocType = 3;
		}
		return edocType;
	}

	/**
	 * 根据模板类型获取govdocType
	 * 
	 * @param moduleType
	 * @return
	 */
	public static Integer getGovdocTypeByModuleType(int moduleType) {
		int govdocType = 1;
		if (moduleType == ApplicationCategoryEnum.govdocSend.getKey()) {
			govdocType = ApplicationSubCategoryEnum.edoc_fawen.getKey();
		} else if (moduleType == ApplicationCategoryEnum.govdocRec.getKey()) {
			govdocType = ApplicationSubCategoryEnum.edoc_shouwen.getKey();
		} else if (moduleType == ApplicationCategoryEnum.govdocExchange.getKey()) {
			govdocType = ApplicationSubCategoryEnum.edoc_jiaohuan.getKey();
		} else if (moduleType == ApplicationCategoryEnum.govdocSign.getKey()) {
			govdocType = ApplicationSubCategoryEnum.edoc_qianbao.getKey();
		}
		return govdocType;
	}

	/**
	 * 根据模板类型获取分送状态
	 * 
	 * @param moduleType
	 * @return
	 */
	public static Integer getTransferStatusByModuleType(int moduleType) {
		Integer transferStatus = TransferStatus.defaultStatus.getKey();
		if (moduleType == ApplicationCategoryEnum.govdocExchange.getKey()) {
			transferStatus = TransferStatus.waitSigned.getKey();
		}
		return transferStatus;
	}

	public static String getNPSAppName(int govdocType, int edocType) {
		String subAppName = ApplicationCategoryEnum.edoc.name();
		if (govdocType == 0) {// 老公文
			if (edocType == 0) {
				subAppName = ApplicationCategoryEnum.edocSend.name();
			} else if (edocType == 1) {
				subAppName = ApplicationCategoryEnum.edocRec.name();
			} else if (edocType == 2) {
				subAppName = ApplicationCategoryEnum.edocSign.name();
			}
		} else {// 新公文
			if (govdocType == 1) {
				subAppName = ApplicationCategoryEnum.govdocSend.name();
			} else if (govdocType == 2) {
				subAppName = ApplicationCategoryEnum.govdocRec.name();
			} else if (govdocType == 3) {
				subAppName = ApplicationCategoryEnum.govdocSign.name();
			} else if (govdocType == 4) {
				subAppName = ApplicationCategoryEnum.govdocExchange.name();
			}
		}
		return subAppName;
	}

	/**
	 * 公文正文类型与字段串互转
	 * 
	 * @param contentType
	 * @return
	 */
	public static String getBodyType(int contentType) {
		String govdocBodyType = new String();
		if (contentType == 10) {
			govdocBodyType = MainbodyType.HTML.name();
		} else if (contentType == 41) {
			govdocBodyType = MainbodyType.OfficeWord.name();
		} else if (contentType == 42) {
			govdocBodyType = MainbodyType.OfficeExcel.name();
		} else if (contentType == 43) {
			govdocBodyType = MainbodyType.WpsWord.name();
		} else if (contentType == 44) {
			govdocBodyType = MainbodyType.WpsExcel.name();
		} else if (contentType == 45) {
			govdocBodyType = MainbodyType.Pdf.name();
			// 流版组件改造
		} else if (contentType == 46) {
			govdocBodyType = MainbodyType.Ofd.name();
		} else {
			govdocBodyType = MainbodyType.OfficeWord.name();
		}
		return govdocBodyType;
	}

	/**
	 * 公文正文类型与字段串互转
	 * 
	 * @param bodyType
	 * @return
	 */
	public static int getContentType(String bodyType) {
		if (MainbodyType.HTML.name().equals(bodyType) || "10".equals(bodyType)) {
			return MainbodyType.HTML.getKey();
		} else if (MainbodyType.OfficeWord.name().equals(bodyType) || "41".equals(bodyType)) {
			return MainbodyType.OfficeWord.getKey();
		} else if (MainbodyType.OfficeExcel.name().equals(bodyType) || "42".equals(bodyType)) {
			return MainbodyType.OfficeExcel.getKey();
		} else if (MainbodyType.WpsWord.name().equals(bodyType) || "43".equals(bodyType)) {
			return MainbodyType.WpsWord.getKey();
		} else if (MainbodyType.WpsExcel.name().equals(bodyType) || "44".equals(bodyType)) {
			return MainbodyType.WpsExcel.getKey();
		} else if (MainbodyType.Pdf.name().equals(bodyType) || "45".equals(bodyType)) {
			return MainbodyType.Pdf.getKey();
		} else if (MainbodyType.Ofd.name().equals(bodyType) || "46".equals(bodyType)) {
			return MainbodyType.Ofd.getKey();
		}
		return MainbodyType.OfficeWord.getKey();
	}

	/**
	 * 公文正文类型与字段串互转
	 * 
	 * @param contentType
	 * @return
	 */
	public static String getBodyTypeText(int contentType) {
		String govdocBodyTypeText = ResourceUtil.getString("govdoc.content.label");
		if (contentType == 41) {
			govdocBodyTypeText = ResourceUtil.getString(MainbodyType.OfficeWord.getText());
		} else if (contentType == 45) {
			govdocBodyTypeText = ResourceUtil.getString(MainbodyType.Pdf.getText());
			// 流版组件改造
		} else if (contentType == 46) {
			govdocBodyTypeText = ResourceUtil.getString(MainbodyType.Ofd.getText());
		}
		return govdocBodyTypeText;
	}

	/**
	 * 判断文单是否为公文单
	 * 
	 * @param formType
	 * @return
	 */
	public static boolean isGovdocForm(int formType) {
		return formType == FormType.govDocSignForm.getKey() || formType == FormType.govDocSendForm.getKey()
				|| formType == FormType.govDocReceiveForm.getKey() || formType == FormType.govDocExchangeForm.getKey();
	}

	/**
	 * 公文文号表单类型名与元素名互转
	 * 
	 * @param markType
	 * @return
	 */
	public static Integer getMarkTypeValueByType(String markType) {
		if ("edocDocMark".equals(markType) || "doc_mark".equals(markType) || "0".equals(markType)) {
			return 0;
		} else if ("edocInnerMark".equals(markType) || "serial_no".equals(markType) || "1".equals(markType)) {
			return 1;
		} else if ("edocSignMark".equals(markType) || "sign_mark".equals(markType) || "2".equals(markType)) {
			return 2;
		}
		return null;
	}

	/**
	 * 按文号长度在流水号前补0
	 * 
	 * @param number
	 * @param length
	 * @return
	 */
	public static String getNumberByFormat(int number, int length) {
		String number_str = "";
		String number_s = String.valueOf(number);
		for (int i = number_s.length(); i < length; i++) {
			number_str += "0";
		}
		number_str += number;
		return number_str;
	}

	/**
	 * 特殊字符转化
	 * 
	 * @param str
	 * @return
	 */
	public static String convertSpecialChat(String str) {
		if (Strings.isNotBlank(str)) {
			StringBuilder buffer = new StringBuilder();
			for (int i = 0; i < str.length(); i++) {
				if (str.charAt(i) == '\'') {
					buffer.append("\\'");
				} else {
					buffer.append(str.charAt(i));
				}
			}
			str = SQLWildcardUtil.escape(buffer.toString());
		}
		return str;
	}

	/**
	 * 后端弹出Alert
	 * 
	 * @param response
	 * @param msg
	 * @throws IOException
	 */
	public static void newCollAlertAndBack(HttpServletResponse response, String msg) throws IOException {
		PrintWriter out = response.getWriter();
		out.println("<script>");
		out.println("alert('" + msg + "');");
		out.print("parent.window.close();");
		out.print("parent.window.history.back();");
		out.println("</script>");
		out.flush();
	}

	/**
	 * 后端弹出Alert
	 * 
	 * @param response
	 * @param msg
	 * @throws IOException
	 */
	public static void newCollAlert(HttpServletResponse response, String msg) throws IOException {
		response.setContentType("text/html;charset=UTF-8");
		PrintWriter out = response.getWriter();
		out.println("<script>");
		out.println("alert('" + msg + "');");
		out.print("parent.window.close();");
		out.println("</script>");
		out.flush();
	}

	/**
	 * 后端弹出Alert
	 * 
	 * @param response
	 * @param msg
	 * @throws IOException
	 *             支持一层父页面
	 */
	public static void webAlertAndClose(HttpServletResponse response, String msg) throws IOException {
		response.setContentType("text/html;charset=UTF-8");
		PrintWriter out = response.getWriter();
		out.println("<script>");
		out.println("alert('" + StringEscapeUtils.escapeJavaScript(msg) + "');");
		out.println("if(window.parent){");
		out.println(" window.parent.close();");
		out.println("}"); // 支持一层父页面
		out.println("if(window.parentDialogObj && window.parentDialogObj['dialogDealColl']){");
		out.println(" window.parentDialogObj['dialogDealColl'].close();");
		out.println("}else if(window.dialogArguments){"); // 弹出
		out.println("  if(window.dialogArguments.parentDialogObj){");
		out.println("    try{window.dialogArguments.parentDialogObj.close();}catch(e){}");
		out.println("  }else{");
		out.println("    window.close();");
		out.println("  }");
		out.println("}else{");
		out.println(" window.close();");
		out.println("}");
		out.println("</script>");
	}

	/**
	 * 后端弹出Alert
	 * 
	 * @param response
	 * @param msg
	 * @param dialogId
	 * @throws IOException
	 */
	public static void webAlertAndCloseDialog(HttpServletResponse response, String msg, String dialogId)
			throws IOException {
		response.setContentType("text/html;charset=UTF-8");
		PrintWriter out = response.getWriter();
		out.println("<script>");
		out.println("alert('" + StringEscapeUtils.escapeJavaScript(msg) + "');");
		out.println("var box = window.parent.document.getElementById('" + dialogId + "');");
		out.println("var mask = window.parent.document.getElementById('" + dialogId + "_mask');");
		out.println("if(mask){mask.parentNode.removeChild(mask);box.parentNode.removeChild(box);}");
		out.println("else if(window.parentDialogObj && window.parentDialogObj['dialogDealColl']){");
		out.println(" window.parentDialogObj['dialogDealColl'].close();");
		out.println("}else if(window.dialogArguments){"); // 弹出
		out.println("  if(window.dialogArguments.parentDialogObj){");
		out.println("    try{window.dialogArguments.parentDialogObj.close();}catch(e){}");
		out.println("  }else{");
		out.println("    window.close();");
		out.println("  }");
		out.println("}else{");
		out.println(" window.close();");
		out.println("}");
		out.println("</script>");
	}

	/**
	 * 替换字符串并让它的下一个字母为大写
	 * 
	 * @param srcStr
	 * @param org
	 * @param ob
	 * @return
	 */
	public static String replaceUnderlineAndfirstToUpper(String srcStr, String org) {
		String newString = "";
		int first = 0;
		while (srcStr.indexOf(org) != -1) {
			first = srcStr.indexOf(org);
			if (first != srcStr.length()) {
				newString = newString + srcStr.substring(0, first);
				srcStr = srcStr.substring(first + org.length(), srcStr.length());
				srcStr = firstCharacterToUpper(srcStr);
			}
		}
		newString = newString + srcStr;
		return newString;
	}

	/**
	 * 首字母大写
	 * 
	 * @param srcStr
	 * @return
	 */
	private static String firstCharacterToUpper(String srcStr) {
		return srcStr.substring(0, 1).toUpperCase() + srcStr.substring(1);
	}

	/**
	 * 根据EdocSummary设置WorkflowData
	 * @param summary
	 * @param flow
	 * @throws BusinessException
	 */
	public static void setWorkflowDataByEdocSummary(EdocSummary summary, WorkflowData flow) throws BusinessException{
		if(summary == null || flow == null){
			log.info("summary == null || flow == null ");
			throw new BusinessException("summary == null || flow == null");
		}
		
		Integer subApp = convertEdocTypeAndEdocGovdocTypeToSubApp(summary.getEdocType(), summary.getGovdocType());
		
		setWorkflowDataBySubApp(subApp, flow);
	}
	/**
	 * 根据公文类型转换成subApp
	 * @param edocType
	 * @param govdocType 必填
	 * @return
	 * @throws BusinessException
	 */
	private static int convertEdocTypeAndEdocGovdocTypeToSubApp(Integer edocType, Integer govdocType) throws BusinessException{
		if(govdocType == null){
			log.info("govdocType  == null ");
			throw new BusinessException("参数错误【govdocType  == null 】");
		}
		
		if(isNewEdocByGovdocType(govdocType)){
			return govdocType;
		}else{
			return converEdocTypeToSubApp(edocType);
		}
	}
	/**
	 * 根据公文类型转换成subApp(老公文)
	 * @param edocType
	 * @return
	 * @throws BusinessException
	 */
	private static int converEdocTypeToSubApp(Integer edocType) throws BusinessException{
		if(edocType == null){
			log.info("edocType  == null ");
			throw new BusinessException("参数错误[edocType  == null]");
		}
		
		if(EdocEnum.edocType.sendEdoc.ordinal() == edocType){
			return EdocEnum.EdocSubApp.edocSendOld.getKey();
		}else if(EdocEnum.edocType.recEdoc.ordinal() == edocType){
			return EdocEnum.EdocSubApp.edocRecOld.getKey();
		}else if(EdocEnum.edocType.signReport.ordinal() == edocType){
			return EdocEnum.EdocSubApp.edocSignOld.getKey();
		}else{
			log.info("edocType = else = null edocType"+edocType);
			throw new BusinessException("参数错误[edocType]"+edocType);
		}
	}
	/**
	 * 根据govdocType判断是否新公文
	 * @param govdocType 必填 EdocSummary.govdocType
	 * @return
	 * @throws BusinessException
	 */
	public static boolean isNewEdocByGovdocType(Integer govdocType) throws BusinessException{
		if(govdocType == null){
			log.info("govdocType 参数 is null :govdocType,"+govdocType);
    		throw new BusinessException("参数错误");
    	}
		
		if(EdocEnum.GovdocType.oldEdoc.getKey() == govdocType){
			return false;
		}else if(EdocEnum.GovdocType.edocSend.getKey() == govdocType ||
				EdocEnum.GovdocType.edocRec.getKey() == govdocType ||
				EdocEnum.GovdocType.edocSign.getKey() == govdocType ||
				EdocEnum.GovdocType.exchange.getKey() == govdocType){
			return true;
		}else{
			log.info("else 参数错误 :govdocType,"+govdocType);
			throw new BusinessException("参数错误");
		}
	}
	
	/**
	 * 根据subApp设置WorkflowData
	 * @param subApp
	 * @param flow
	 * @throws BusinessException
	 */
	public static void setWorkflowDataBySubApp(Integer subApp, WorkflowData flow) throws BusinessException{
		if(subApp == null || flow == null){
			log.info("subApp == null || flow == null ");
			throw new BusinessException("参数错误[subApp == null || flow == null]");
		}
		
		if(EdocEnum.EdocSubApp.edocSend.getKey() == subApp){
			flow.setAppType(ResourceUtil.getString("edoc.docmark.inner.send"));
			flow.setEdocType(FormType.govDocSendForm.getKey());
			flow.setNps(ApplicationCategoryEnum.govdocSend.name());
		}else if(EdocEnum.EdocSubApp.edocRec.getKey() == subApp){
			flow.setAppType(ResourceUtil.getString("edoc.docmark.inner.receive"));
			flow.setEdocType(FormType.govDocReceiveForm.getKey());
			flow.setNps(ApplicationCategoryEnum.govdocRec.name());
		}else if(EdocEnum.EdocSubApp.edocSign.getKey() == subApp){
			flow.setAppType(ResourceUtil.getString("edoc.docmark.inner.signandreport"));
			flow.setEdocType(FormType.govDocSignForm.getKey());
			flow.setNps(ApplicationCategoryEnum.govdocSign.name());
		}if(EdocEnum.EdocSubApp.edocSendOld.getKey() == subApp){
			flow.setAppType(ResourceUtil.getString("edoc.docmark.inner.send"));
    		flow.setEdocType(FormType.govDocSendForm.getKey());
    		flow.setNps(ApplicationCategoryEnum.edocSend.name());
		}else if(EdocEnum.EdocSubApp.edocRecOld.getKey() == subApp){
			flow.setAppType(ResourceUtil.getString("edoc.docmark.inner.receive"));
    		flow.setEdocType(FormType.govDocReceiveForm.getKey());
    		flow.setNps(ApplicationCategoryEnum.edocRec.name());
		}else if(EdocEnum.EdocSubApp.edocSignOld.getKey() == subApp){
			flow.setAppType(ResourceUtil.getString("edoc.docmark.inner.signandreport"));
    		flow.setEdocType(FormType.govDocSignForm.getKey());
    		flow.setNps(ApplicationCategoryEnum.edocSign.name());
		}
		flow.setAppEnumStr(ApplicationCategoryEnum.edoc.name());
	}
	
	/**
	 * SQL脚本处理
	 * 
	 * @param str
	 * @return
	 */
	public static String getSQLStr(String str) {
		if (Strings.isNotBlank(str)) {
			str = str.replaceAll(String.valueOf((char) 160), String.valueOf((char) 32));
			str = SQLWildcardUtil.escape(str);
		}
		return str;
	}

	public static List<Long> getIdList(String idstr) {
		List<Long> idList = new ArrayList<Long>();
		if (Strings.isNotBlank(idstr)) {
			String[] idArr = idstr.split(",");
			for (String string : idArr) {
				if (Strings.isNotBlank(string)) {
					idList.add(Long.valueOf(string));
				}
			}
		}
		return idList;
	}

	public static Map<String, String> cloneMap(Map<String, String> sourceMap) {
		if (sourceMap == null) {
			return null;
		}
		Set<String> keys = sourceMap.keySet();
		Map<String, String> newMap = new HashMap<String, String>();
		for (String key : keys) {
			newMap.put(key, sourceMap.get(key));
		}
		return newMap;
	}

	/**
	 * List分页
	 * 
	 * @param targe
	 * @param size
	 * @return
	 */
	public static List<List<Long>> createList(List<Long> targe, int size) {
		List<List<Long>> listArr = new ArrayList<List<Long>>();
		// 获取被拆分的数组个数
		int arrSize = targe.size() % size == 0 ? targe.size() / size : targe.size() / size + 1;
		for (int i = 0; i < arrSize; i++) {
			List<Long> sub = new ArrayList<Long>();
			// 把指定索引数据放入到list中
			for (int j = i * size; j <= size * (i + 1) - 1; j++) {
				if (j <= targe.size() - 1) {
					sub.add(targe.get(j));
				}
			}
			listArr.add(sub);
		}
		return listArr;
	}

	/**
	 * 设置govdocExchangeDetail扩展字段值
	 */
	public static void setExtProperty(GovdocExchangeDetail detail, Map<String, Object> map) {
		if (map != null) {
			detail.setExtAttr(XMLCoder.encoder(map));
		}
	}

	/**
	 * 获取govdocExchangeDetail扩展字段值
	 */
	@SuppressWarnings("unchecked")
	public static Map<String, Object> getExtProperty(GovdocExchangeDetail detail) {
		return (Map<String, Object>) (Strings.isBlank(detail.getExtAttr()) ? new HashMap<String, Object>()
				: (Map<String, Object>) XMLCoder.decoder(detail.getExtAttr()));
	}

	public static boolean isExchangeSendPendingList(String listType) {
		return "listExchangeSendPending".equals(listType);
	}

	public static boolean isExchangeSignList(String listType) {
		return ("listExchangeSignDone".equals(listType) || "listExchangeFallback".equals(listType) ||  "listExchangeReturnedFallback".equals(listType)
				|| "listExchangeSignPending".equals(listType));
	}

	public static boolean isGovdocWfOld(Integer app, Integer subApp) {
		return app == ApplicationCategoryEnum.edoc.key()
				&& (subApp.intValue() == ApplicationSubCategoryEnum.old_edocSend.key()
						|| subApp.intValue() == ApplicationSubCategoryEnum.old_edocRec.key()
						|| subApp.intValue() == ApplicationSubCategoryEnum.old_edocSign.key());
	}

	public static boolean isGovdocWf(Integer app, Integer subApp) {
		return app == ApplicationCategoryEnum.edoc.key()
				&& (subApp.intValue() == ApplicationSubCategoryEnum.edoc_fawen.key()
						|| subApp.intValue() == ApplicationSubCategoryEnum.edoc_qianbao.key()
						|| subApp.intValue() == ApplicationSubCategoryEnum.edoc_shouwen.key()
						|| subApp.intValue() == ApplicationSubCategoryEnum.edoc_jiaohuan.key());
	}

	public static List<Integer> getSubAppByEdocType(int edocType) {
		List<Integer> subApp = new ArrayList<Integer>();
		if (edocType == 0) {
			subApp.add(ApplicationSubCategoryEnum.edoc_fawen.key());
			subApp.add(ApplicationSubCategoryEnum.old_edocSend.key());
			// subApp.add(ApplicationSubCategoryEnum.old_exSend.key());
		} else if (edocType == 1) {
			subApp.add(ApplicationSubCategoryEnum.edoc_shouwen.key());
			subApp.add(ApplicationSubCategoryEnum.old_edocRec.key());
			// subApp.add(ApplicationSubCategoryEnum.old_exSign.key());
			// subApp.add(ApplicationSubCategoryEnum.old_edocRecDistribute.key());
		} else if (edocType == 2) {
			subApp.add(ApplicationSubCategoryEnum.edoc_qianbao.key());
			subApp.add(ApplicationSubCategoryEnum.old_edocSign.key());
		}
		return subApp;
	}

	public static boolean isH5() {
		Object tag = AppContext.getThreadContext(FormConstant.h5Tag);
		boolean h5Tag = (tag == null) ? false : (Boolean) tag;// 预留h5参数
		return h5Tag;
	}

	public static void webAlertAndCloseComponentDiv(HttpServletResponse response, String msg) throws IOException {
		response.setContentType("text/html;charset=UTF-8");
		response.resetBuffer();
		PrintWriter out = response.getWriter();
		out.println("<script>");
		out.println("alert(\'" + StringEscapeUtils.escapeJavaScript(msg) + "\');");
		out.println(" window.parent.close();");
		out.println("</script>");
	}

	/**
	 * 获得节点权限名称
	 * 
	 * @param affair
	 * @return
	 * @throws BusinessException
	 */
	public static SeeyonPolicy getPolicyByAffair(CtpAffair affair) throws BusinessException {
		try {
			SeeyonPolicy seeyonPolicy = getPolicy(affair);
			if (seeyonPolicy == null) {
				return new SeeyonPolicy("shenpi", ResourceUtil.getString("node.policy.shenpi"));
			} else if (Strings.isBlank(seeyonPolicy.getId())) {
				return new SeeyonPolicy("shenpi", ResourceUtil.getString("node.policy.shenpi"));
			}
			return seeyonPolicy;
		} catch (Exception e) {
			log.error("", e);
			return new SeeyonPolicy("shenpi", ResourceUtil.getString("node.policy.shenpi"));
		}
	}

	/**
	 * 创建一个没有态度的空意见，只有处理人、处理时间
	 * 
	 * @param affair
	 * @param summary
	 * @param extParam
	 * @return
	 */

	public static Comment createNullCommentWithoutAttitude(CtpAffair affair, Long summaryId) {

		Comment comment = new Comment();

		comment.setHidden(false);
		comment.setId(UUIDLong.longUUID());

		comment.setCtype(Comment.CommentType.comment.getKey());
		comment.setPid(0L);
		comment.setPraiseNumber(0);
		comment.setPraiseToComment(false);
		comment.setPraiseToComment(false);
		comment.setModuleType(ModuleType.edoc.getKey());
		comment.setModuleId(summaryId);
		comment.setCreateDate(new Timestamp(System.currentTimeMillis()));
		comment.setAffairId(affair.getId());
		comment.setCreateId(affair.getMemberId());
		comment.setForwardCount(0);
		comment.setPath(AppContext.getCurrentUser().getUserAgentFrom() == null ? "pc"
				: AppContext.getCurrentUser().getUserAgentFrom());

		comment.setDepartmentId(affair.getMatchDepartmentId());
		comment.setPostId(affair.getMatchPostId());
		comment.setAccountId(affair.getMatchAccountId());

		return comment;
	}

	private static SeeyonPolicy getPolicy(CtpAffair affair) throws BusinessException, BPMException {
		if (affair == null || affair.getState().equals(StateEnum.col_waitSend.getKey())
				|| affair.getState().equals(StateEnum.col_sent.getKey())) {
			if (affair != null && ApplicationSubCategoryEnum.edoc_shouwen.getKey() == affair.getSubApp()) {
				return new SeeyonPolicy("dengji", ResourceUtil.getString("node.policy.dengji"));
			} else {
				return new SeeyonPolicy("niwen", ResourceUtil.getString("node.policy.niwen"));
			}

		}
		if (Strings.isNotBlank(affair.getNodePolicy())) {
			return new SeeyonPolicy(affair.getNodePolicy(), BPMSeeyonPolicy.getShowName(affair.getNodePolicy()));
		}
		WorkflowApiManager wapi = getWorkflowApiManager();
		EdocSummaryManager edocSummaryManager = getEdocSummaryManager();
		EdocSummary summary = edocSummaryManager.getEdocSummaryById(affair.getObjectId(), false, false);
		if (summary != null) {
			String[] result = wapi.getNodePolicyIdAndName(ModuleType.edoc.name(), summary.getProcessId(),
					String.valueOf(affair.getActivityId()));
			if (result == null)
				return new SeeyonPolicy("shenpi", ResourceUtil.getString("node.policy.shenpi"));
			return new SeeyonPolicy(result[0], result[1]);
		} else {
			return new SeeyonPolicy("shenpi", ResourceUtil.getString("node.policy.shenpi"));
		}
	}

	private static WorkflowApiManager getWorkflowApiManager() {
		if (wapi == null) {
			wapi = (WorkflowApiManager) AppContext.getBean("wapi");
		}
		return wapi;
	}

	public static OrgManager getOrgManager() {
		if (orgManager == null) {
			orgManager = (OrgManager) AppContext.getBean("orgManager");
		}
		return orgManager;
	}

	private static EdocSummaryManager getEdocSummaryManager() {
		if (edocSummaryManager == null) {
			edocSummaryManager = (EdocSummaryManager) AppContext.getBean("edocSummaryManager");
		}
		return edocSummaryManager;
	}
	
	public static EnumNameEnum getEdocMetadataNameEnum(int govdocType)
	{
		if(govdocType == ApplicationSubCategoryEnum.edoc_fawen.key()){
			return EnumNameEnum.edoc_new_send_permission_policy;
		}else if(govdocType== ApplicationSubCategoryEnum.edoc_shouwen.key()){
			return EnumNameEnum.edoc_new_rec_permission_policy;
		}else if(govdocType== ApplicationSubCategoryEnum.edoc_qianbao.key()){
			return EnumNameEnum.edoc_qianbao_permission_policy;
		}else{
			return EnumNameEnum.edoc_new_send_permission_policy;
		}
	}
	
	public static void setHasAttachments(EdocSummary summary, Boolean hasAttachments) {
		if (Strings.isBlank(summary.getIdentifier())) {
			summary.setIdentifier(IdentifierUtil.newIdentifier(summary.getIdentifier(), INENTIFIER_SIZE, '0'));
		}
		summary.setIdentifier(IdentifierUtil.update(summary.getIdentifier(), INENTIFIER_INDEX.HAS_ATTACHMENTS.ordinal(),
				hasAttachments ? '1' : '0'));
	}

	/**
	 * 标志位, 共100位，采用枚举的自然顺序
	 */
	protected static enum INENTIFIER_INDEX {
		HAS_ATTACHMENTS, // 是否有附件
		HAS_FORMTRIGGER, // 是否有配置表单触发
		IS_WORKFLOWTRACE // 是否追溯克隆数据
	}

	public static String getMemberName(Long id) {
		return Functions.showMemberName(id);
	}

	/**
	 * 判定给出的字符串是否为数字格式的（含负数的情况(-)） 创建人:zhiyanqiang
	 * 对功能的增强：现在解析文件Id时不规范，添加该校验，增强容错性 创建时间：2016年4月22日 下午4:11:39
	 * 
	 * @param str
	 * @return boolean
	 */
	public static boolean isNumeric(String str) {
		Pattern pattern = Pattern.compile("-?[0-9]+");
		Matcher isNum = pattern.matcher(str);
		if (!isNum.matches()) {
			return false;
		}
		return true;
	}

	public static String getFullNameByMultiAccount(String accountOrDepartmentId) {
		try {
			String finalValue = "";
			OrgManager orgManager = getOrgManager();
			V3xOrgUnit entity = orgManager.getUnitById(Long.valueOf(accountOrDepartmentId));
			if (entity != null) {
				finalValue = entity.getName() + "hiddenValue" + entity.getType() + "|" + accountOrDepartmentId;
			}
			return finalValue;
		} catch (Exception e) {
			log.error("GetFullNameByMultiAccount error ： id is " + accountOrDepartmentId + e.getMessage());
		}
		return "";
	}

	public static void saveGovdocExtendAndSort(FormBean fb,GovdocFormExtendManager govdocFormExtendManager,GovdocFormOpinionSortManager govdocFormOpinionSortManager){
		List<GovdocFormOpinionSort> govdocFormOpinionSorts = govdocFormOpinionSortManager.findByFormId(fb.getId());
        GovdocFormExtend govdocFormExtend = govdocFormExtendManager.findByFormId(fb.getId());
        if(govdocFormExtend==null){
        	govdocFormExtend = new GovdocFormExtend();
        	govdocFormExtend.setAccountId(AppContext.currentAccountId());
        	govdocFormExtend.setId(UUIDLong.longUUID());
        	govdocFormExtend.setFormId(fb.getId());
        	govdocFormExtend.setOptionFormatSet(FormOpinionConfig.getDefualtConfig());
        	govdocFormExtendManager.saveOrUpdate(govdocFormExtend);
        }
        if(null==govdocFormOpinionSorts||govdocFormOpinionSorts.size()<=0){
        	List<FormFieldBean> formFieldBeans = fb.getAllFieldBeans();
        	govdocFormOpinionSorts = new ArrayList<GovdocFormOpinionSort>();
            for (FormFieldBean formFieldBean : formFieldBeans) {
				if("edocflowdealoption".equals(formFieldBean.getInputType())){
					GovdocFormOpinionSort govdocFormOpinionSort = new GovdocFormOpinionSort();
					govdocFormOpinionSort.setId(UUIDLong.longUUID());
					govdocFormOpinionSort.setDomainId(AppContext.currentAccountId());
					govdocFormOpinionSort.setProcessName(formFieldBean.getName());
					govdocFormOpinionSort.setFormId(fb.getId());
					govdocFormOpinionSorts.add(govdocFormOpinionSort);
				}
			}
            govdocFormOpinionSortManager.saveOrUpdateList(govdocFormOpinionSorts);
        }else{
        	String fieldNum = "";
        	boolean isOldOpinionField = false;
        	for(int i=0;i<govdocFormOpinionSorts.size();i++){
        		fieldNum = fieldNum + govdocFormOpinionSorts.get(i).getProcessName()+",";
        	}
        	List<FormFieldBean> formFieldBeans = fb.getAllFieldBeans();
        	List<GovdocFormOpinionSort> govdocFormOpinionSorts2 = new ArrayList<GovdocFormOpinionSort>();
        	for (FormFieldBean formFieldBean : formFieldBeans) {
				if("edocflowdealoption".equals(formFieldBean.getInputType())){
					GovdocFormOpinionSort govdocFormOpinionSort1 = new GovdocFormOpinionSort();
					govdocFormOpinionSort1.setId(UUIDLong.longUUID());
					govdocFormOpinionSort1.setDomainId(AppContext.currentAccountId());
					govdocFormOpinionSort1.setProcessName(formFieldBean.getName());
					govdocFormOpinionSort1.setFormId(fb.getId());
					for(GovdocFormOpinionSort govdocFormOpinionSort:govdocFormOpinionSorts){
						if(govdocFormOpinionSort.getFormId()==fb.getId()&&govdocFormOpinionSort.getProcessName().equals(formFieldBean.getName())){
							govdocFormOpinionSort1.setSortType(govdocFormOpinionSort.getSortType());
						}
					}
					govdocFormOpinionSorts2.add(govdocFormOpinionSort1);
				}else if(!"edocflowdealoption".equals(formFieldBean.getInputType()) && fieldNum.contains(formFieldBean.getName())){
					isOldOpinionField = true;
				}
			}
        	if(isOldOpinionField && (govdocFormOpinionSorts2==null || govdocFormOpinionSorts2.size()==0)){
        		govdocFormOpinionSortManager.deleteByFormId(fb.getId());
        	}else{
        		govdocFormOpinionSortManager.saveOrUpdateList(govdocFormOpinionSorts2);	
        	}
        }
	}
}
